# encoding: utf-8
#
# Cookbook Name:: pki
# Recipe:: server_pki
#
# Author:: Sean OMeara <someara@opscode.com>
#
# Copyright 2012, Opscode, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Common packages
package "openssl"
package "rsync"
package "xinetd"

# create CA if one does not exist
pki_selfsignedca "ca" do
  action [:create]
end

# make a directory to place requests in
directory "/etc/pki/CA/requests" do
  action [:create]
  mode 0700
end

# rsync xinetd configuration
template "/etc/xinetd.d/rsync" do
  source "rsync.xinetd.erb"
  notifies :restart, "service[xinetd]"
end

# rsyncd config
template "/etc/rsyncd.conf" do
  source "rsyncd.conf.erb"
end

# ensure xinetd is running
service "xinetd" do
  action [:enable,:start]
end

# poke a hole in the firewall
# FIXME Do this properly once COOK-688 is done
service "iptables" do
  action [:disable,:stop]
end

#### 
# create self client certificates.
#### 

# key
ruby_block "generate private key" do
  block do
    Chef::Log.info("creating servercert private key for #{node[:fqdn]}")
    system("openssl genrsa -out /etc/pki/tls/private/#{node[:fqdn]}.key 2048")
  end
  not_if { ::File.exists?("/etc/pki/tls/private/#{node[:fqdn]}.key") }
end.run_action(:create)

# If key but no cert, generate CSR
ruby_block "generating CSR" do
  block do
    Chef::Log.info("generating certificate request")
    cmd = "openssl req"
    cmd += " -new"
    cmd += " -nodes"
    cmd += " -subj '/CN=#{node[:fqdn]}/'"
    cmd += " -key /etc/pki/tls/private/#{node[:fqdn]}.key"
    cmd += " -out /etc/pki/tls/private/#{node[:fqdn]}.csr"
    system(cmd)
  end
  not_if { ::File.exists?("/etc/pki/tls/private/#{node[:fqdn]}.csr") }
  only_if { ::File.exists?("/etc/pki/tls/private/#{node[:fqdn]}.key") }
end

# Copy the request into the CA
ruby_block "copy server's request into the CA" do
  block do
    FileUtils.cp("/etc/pki/tls/private/#{node[:fqdn]}.csr","/etc/pki/CA/requests/#{node[:fqdn]}.csr")
  end
  only_if { ::File.exists?("/etc/pki/tls/private/#{node[:fqdn]}.csr") }
  not_if { ::File.exists?("/etc/pki/CA/requests/#{node[:fqdn]}.csr") }
end

# sign the certificate request.
ruby_block "signing the server's certificate request" do
  block do
    cmd = "openssl x509"
    cmd += " -req"
    cmd += " -CA /etc/pki/CA/certs/ca.crt"
    cmd += " -CAkey /etc/pki/CA/private/ca.key"
    cmd += " -in /etc/pki/CA/requests/#{node[:fqdn]}.csr"
    cmd += " -out /etc/pki/CA/certs/#{node[:fqdn]}.crt"
    system(cmd)
  end
  not_if { ::File.exists?("/etc/pki/tls/private/#{node[:fqdn]}.crt") }
  only_if { ::File.exists?("/etc/pki/CA/requests/#{node[:fqdn]}.csr") }
end

# Ship it.
ruby_block "copy the server's certificate from the CA into the tls dir" do
  block do
    FileUtils.cp("/etc/pki/CA/certs/#{node[:fqdn]}.crt","/etc/pki/tls/private/#{node[:fqdn]}.crt")
  end
  only_if { ::File.exists?("/etc/pki/CA/certs/#{node[:fqdn]}.crt") }
  not_if { ::File.exists?("/etc/pki/tls/private/#{node[:fqdn]}.crt") }
end

